#!/usr/bin/env python
# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Distutils installer for MAAS."""

# unicode_literals is *not* included here to *better* support setup
# scripts that work on both Python 2 and 3. Both expect their default
# string type, and are fussy about it.
from __future__ import (
    absolute_import,
    print_function,
    )

__version__ = "1.4"
__metaclass__ = type


from codecs import open
from contextlib import contextmanager
from distutils import (
    errors,
    log,
    )
from inspect import getsourcefile
from operator import methodcaller
from os import environ

import setuptools


@contextmanager
def envvar(name, value):
    """Context manager for temporarily setting an environment variable."""
    original = environ.get(name)
    environ[name] = value
    try:
        yield
    finally:
        if original is None:
            environ.pop(name, None)
        else:
            environ[name] = original


class install_deps(setuptools.Command):
    """Distutils/setuptools command to install package dependencies."""

    user_options = [
        ('list-from=', None,
         "file from which to read packages to install"),
    ]

    def initialize_options(self):
        self.list_from = "packages.txt"

    def gen_package_ops(self, package_lines):
        install = methodcaller("mark_install")
        delete = methodcaller("mark_delete", purge=True)
        for line in package_lines:
            if len(line) == 0:
                pass  # Ignore blank lines.
            elif line.startswith("#"):
                pass  # Ignore whole-line comments.
            elif line.endswith("-"):
                package_name = line.rstrip("-")
                yield package_name, delete
            else:
                package_name = line.rstrip("+")
                yield package_name, install

    def finalize_options(self):
        self.ensure_filename("list_from")
        self.packages = set()
        with open(self.list_from, "r", encoding="utf-8") as fd:
            lines = (line.strip() for line in fd)
            self.package_ops = dict(self.gen_package_ops(lines))
            self.packages = set(self.package_ops)

    def run(self):
        self.calculate_changes()
        if len(self.changes) > 0:
            self.apply_changes()

    def calculate_changes(self):
        from apt import Cache
        cache = Cache()
        for package_name, op in self.package_ops.iteritems():
            op(cache[package_name])
        self.changes = cache.get_changes()

    def get_packages_to_install(self):
        return self.packages.intersection(
            package.name for package in self.changes
            if package.marked_install or package.marked_upgrade)

    def get_packages_to_delete(self):
        return self.packages.intersection(
            package.name for package in self.changes
            if package.marked_delete)

    def apply_changes(self):
        """Use ``apt`` to check for package installs.

        Spawns ``apt-get`` if necessary.
        """
        to_install = {
            "%s+" % package
            for package in self.get_packages_to_install()
        }
        if len(to_install) == 0:
            log.info("system dependencies are all in place")

        to_delete = {
            "%s-" % package
            for package in self.get_packages_to_delete()
        }
        if len(to_delete) == 0:
            log.info("no forbidden system dependencies are installed")

        to_do = sorted(to_install | to_delete)
        if len(to_do) > 0:
            install = [
                "sudo", "apt-get", "--assume-yes",
                "--no-install-recommends", "--purge",
                "install",
            ]
            with envvar("DEBIAN_FRONTEND", "noninteractive"):
                self.spawn(install + to_do)


def check_settings(settings):
    """Check setup settings for obvious issues.

    - ``data_files`` should not be specified; setuptools ignores it.

    - ``include_package_data`` should not be specified; it has weird
      behaviour in setuptools.
    """
    if "data_files" in settings:
        raise errors.DistutilsOptionError(
            "data_files is ignored by setuptools")
    if "include_package_data" in settings:
        raise errors.DistutilsOptionError(
            "include_package_data does not work as you "
            "might expect with setuptools")


def setup(**kwargs):
    """MAAS-specific setup function.

    Determines sensible defaults for many parameters.
    """
    command_classes = {}

    try:
        import apt
    except ImportError:
        log.warn("apt not available (install 'python-apt'?)")
    else:
        command_classes["install_deps"] = install_deps
        apt  # Silence lint.

    settings = {
        "cmdclass": command_classes,
        "packages": setuptools.find_packages(".", exclude={"common"}),
        "version": __version__,
        "maintainer": "Canonical",
        "maintainer_email": "maas-devel@lists.launchpad.net",
        "url": "https://launchpad.net/maas",
        "license": "AGPLv3",
        "test_loader": "maastesting.loader:MAASTestLoader",
        "test_suite": ".",
        "tests_require": {"maas-testing >= %s" % __version__},
    }
    settings.update(kwargs)
    check_settings(settings)

    return setuptools.setup(**settings)


def configure(**settings):
    """Create a new ``setup.py`` file in the currnet directory.

    Uses the file in which this function is originally defined as a
    template, adding on a call to ``setup`` with the given ``settings``.
    """
    check_settings(settings)
    thisfile = getsourcefile(configure)
    with open(thisfile, "r", encoding="utf-8") as fin:
        template = fin.read()
    with open("setup.py", "w", encoding="utf-8") as fout:
        fout.write(template)
        fout.write(b"\n\nsetup(**%r)\n" % settings)


setup(**{'test_loader': 'maastesting.djangoloader:MAASDjangoTestLoader', 'entry_points': {'console_scripts': set(['maas-region-worker = maas.entrypoints:celeryd', 'maas-region-twistd = maas.entrypoints:twistd', 'maas = maas.entrypoints:manage', 'maas-dns-server = maas.entrypoints:dns_server'])}, 'name': 'maas-region', 'install_requires': ['amqplib >= 1.0.2', 'celery >= 2.5.3', 'convoy >= 0.2.1', 'crochet >= 1.0.0', 'django >= 1.5.4', 'django-debug-toolbar >= 1.0.1', 'django-piston >= 0.2.3', 'djorm-ext-pgarray >= 0.8', 'docutils >= 0.11', 'formencode >= 1.2.4', 'iscpy >= 1.05', 'jsonschema >= 2.3.0', 'lockfile >= 0.8', 'lxml >= 3.2.0', 'maas-apiclient >= 1.4', 'maas-cluster >= 1.4', 'netaddr >= 0.7.7', 'oauth >= 1.0.1', 'oops >= 0.0.10', 'oops-datedir-repo >= 0.0.15', 'oops-twisted >= 0.0.6', 'psycopg2 >= 2.4.5', 'pyOpenSSL >= 0.13', 'PyYAML >= 3.1.0', 'simplejson >= 3.3.0', 'south >= 0.7.5', 'tempita >= 0.5.1', 'twisted >= 13.0.0', 'txlongpoll >= 0.3.1'], 'tests_require': ['fixtures >= 0.3.14', 'maas-testing >= 1.4', 'mock >= 1.0.1', 'nose >= 1.1.2', 'postgresfixture >= 0.2.1', 'python-subunit >= 0.0.18', 'rabbitfixture >= 0.3.5', 'saucelabsfixture >= 0.1', 'selenium >= 2.39', 'sst >= 0.2.2', 'testresources >= 0.2.5', 'testscenarios >= 0.3', 'testtools >= 0.9.32'], 'description': 'MAAS Region Controller.'})
